from flask import Blueprint, current_app, request
from scmsrht.access import UserAccess
from scmsrht.blueprints.api import get_user, get_repo
from scmsrht.repos import RepoVisibility
from scmsrht.types import Access, Repository, User
from scmsrht.webhooks import UserWebhook
from srht.api import paginated_response
from srht.database import db
from srht.oauth import current_token, oauth
from srht.validation import Validation
from sqlalchemy import and_, or_

info = Blueprint("scm.api.info", __name__)

@info.route("/api/repos", defaults={"username": None})
@info.route("/api/<username>/repos")
@oauth("info:read")
def repos_by_user_GET(username):
    user = get_user(username)
    repos = Repository.query.filter(Repository.owner_id == user.id)
    if user.id != current_token.user_id:
        repos = (repos
                .outerjoin(Access._get_current_object(),
                    Access.repo_id == Repository.id)
                .filter(or_(
                    Access.user_id == current_token.user_id,
                    and_(
                        Repository.visibility == RepoVisibility.public,
                        Access.id.is_(None))
                )))
    return paginated_response(Repository.id, repos)

@info.route("/api/repos", methods=["POST"])
@oauth("info:write")
def repos_POST():
    valid = Validation(request)
    repo = current_app.repo_api.create_repo(valid, current_token.user)
    if not valid.ok:
        return valid.response
    UserWebhook.deliver(UserWebhook.Events.repo_create,
            repo.to_dict(), UserWebhook.Subscription.user_id == repo.owner_id)
    return repo.to_dict(), 201

@info.route("/api/repos/<reponame>", defaults={"username": None})
@info.route("/api/<username>/repos/<reponame>")
@oauth("info:read")
def repos_by_name_GET(username, reponame):
    user = get_user(username)
    repo = get_repo(user, reponame)
    return repo.to_dict()

def _webhook_filters(query):
    user = get_user(None)
    return query.filter(UserWebhook.Subscription.user_id == user.id)

def _webhook_create(sub, valid):
    user = get_user(None)
    return sub

UserWebhook.api_routes(info, "/api/user",
        filters=_webhook_filters, create=_webhook_create)

@info.route("/api/repos/<reponame>", methods=["PUT"])
@oauth("info:write")
def repos_by_name_PUT(reponame):
    user = get_user(None)
    repo = get_repo(user, reponame, needs=UserAccess.manage)

    valid = Validation(request)
    repo.description = valid.optional("description", default=repo.description)
    repo.visibility = valid.optional("visibility",
            cls=RepoVisibility, default=repo.visibility.value)
    name = valid.optional("name", default=repo.name)
    if not valid.ok:
        return valid.response

    if name != repo.name:
        repo = current_app.repo_api.rename_repo(repo.owner, repo, valid)

    if not valid.ok:
        return valid.response

    UserWebhook.deliver(UserWebhook.Events.repo_update,
            repo.to_dict(), UserWebhook.Subscription.user_id == repo.owner_id)

    db.session.commit()
    return repo.to_dict()

@info.route("/api/repos/<reponame>", methods=["DELETE"])
@oauth("info:write")
def repos_by_name_DELETE(reponame):
    user = get_user(None)
    repo = get_repo(user, reponame, needs=UserAccess.manage)
    repo_id = repo.id
    current_app.repo_api.delete_repo(repo)
    UserWebhook.deliver(UserWebhook.Events.repo_delete, { "id": repo_id },
            UserWebhook.Subscription.user_id == repo.owner_id)
    return {}, 204
