import os.path
import pygments
from datetime import timedelta
from jinja2 import Markup, Template
from pygments import highlight
from pygments.lexers import guess_lexer, guess_lexer_for_filename, TextLexer
from pygments.formatters import HtmlFormatter
from srht.markdown import markdown
from scmsrht.redis import redis

def get_formatted_readme(cache_prefix, file_finder, content_getter,
        link_prefix=None):
    readme_names = ['README.md', 'README.markdown', 'README']
    for name in readme_names:
        content_hash, user_obj = file_finder(name)
        if content_hash:
            return format_readme(cache_prefix, content_hash, name,
                        content_getter, user_obj, link_prefix=link_prefix)
    return None

def format_readme(cache_prefix, content_hash, name, content_getter, user_obj,
        link_prefix=None):
    """ Formats a `README` file for display on a repository's summary page.
    """
    key = f"{cache_prefix}:readme:{content_hash}:v6"
    html = redis.get(key)
    if html:
        return Markup(html.decode())

    try:
        raw = content_getter(user_obj)
    except:
        raw = "Error decoding readme - is it valid UTF-8?"

    basename, ext = os.path.splitext(name)
    if ext in ['.md', '.markdown']:
        html = markdown(raw, ["h1", "h2", "h3", "h4", "h5"],
                link_prefix=link_prefix)
    else:
        # Unsupported/unknown markup type.
        html = Template(
                 "<pre>{{ readme }}</pre>",
                 autoescape=True
               ).render(readme=raw)

    redis.setex(key, timedelta(days=7), html)
    return Markup(html)

def _get_shebang(data):
    if not data.startswith('#!'):
        return None

    endline = data.find('\n')
    if endline == -1:
        shebang = data
    else:
        shebang = data[:endline]

    return shebang

def _get_lexer(name, data):
    try:
        return guess_lexer_for_filename(name, data)
    except pygments.util.ClassNotFound:
        try:
            shebang = _get_shebang(data)
            if not shebang:
                return TextLexer()

            return guess_lexer(shebang)
        except pygments.util.ClassNotFound:
            return TextLexer()

def get_highlighted_file(cache_prefix, name, content_hash,
        content, formatter=None):
    """ Highlights a file for display in a repository's browsing UI.
    """
    key = f"{cache_prefix}:highlight:{content_hash}"
    html = redis.get(key)
    if html:
        return Markup(html.decode())

    lexer = _get_lexer(name, content)
    if formatter is None:
        formatter = HtmlFormatter()
    style = formatter.get_style_defs('.highlight')
    html = f"<style>{style}</style>" + highlight(content, lexer, formatter)
    redis.setex(key, timedelta(days=7), html)
    return Markup(html)
